package blusunrize.immersiveengineering.common.items;

import blusunrize.immersiveengineering.api.Lib;
import blusunrize.immersiveengineering.api.TargetingInfo;
import blusunrize.immersiveengineering.api.energy.wires.IImmersiveConnectable;
import blusunrize.immersiveengineering.api.energy.wires.IWireCoil;
import blusunrize.immersiveengineering.api.energy.wires.ImmersiveNetHandler;
import blusunrize.immersiveengineering.api.energy.wires.ImmersiveNetHandler.Connection;
import blusunrize.immersiveengineering.api.energy.wires.WireType;
import blusunrize.immersiveengineering.common.IESaveData;
import blusunrize.immersiveengineering.common.util.IEAchievements;
import blusunrize.immersiveengineering.common.util.ItemNBTHelper;
import blusunrize.immersiveengineering.common.util.Utils;
import net.minecraft.block.state.IBlockState;
import net.minecraft.client.resources.I18n;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Vec3d;
import net.minecraft.util.text.TextComponentTranslation;
import net.minecraft.world.World;

import java.util.HashSet;
import java.util.List;
import java.util.Set;


public class ItemWireCoil extends ItemIEBase implements IWireCoil
{
	public ItemWireCoil()
	{
		super("wirecoil", 64, "copper", "electrum", "hv", "rope", "structural", "redstone");
	}

	@Override
	public WireType getWireType(ItemStack stack)
	{
		switch(stack.func_77952_i())
		{
			case 0:
			default:
				return WireType.COPPER;
			case 1:
				return WireType.ELECTRUM;
			case 2:
				return WireType.STEEL;
			case 3:
				return WireType.STRUCTURE_ROPE;
			case 4:
				return WireType.STRUCTURE_STEEL;
			case 5:
				return WireType.REDSTONE;
		}
	}

	@Override
	public void func_77624_a(ItemStack stack, EntityPlayer player, List list, boolean adv)
	{
		if(stack.func_77952_i() == 5)
		{
			list.add(I18n.func_135052_a(Lib.DESC_FLAVOUR + "coil.redstone"));
			list.add(I18n.func_135052_a(Lib.DESC_FLAVOUR + "coil.construction1"));
		} else if(stack.func_77952_i() > 2)
		{
			list.add(I18n.func_135052_a(Lib.DESC_FLAVOUR+"coil.construction0"));
			list.add(I18n.func_135052_a(Lib.DESC_FLAVOUR+"coil.construction1"));
		}
		if(stack.func_77978_p()!=null && stack.func_77978_p().func_74764_b("linkingPos"))
		{
			int[] link = stack.func_77978_p().func_74759_k("linkingPos");
			if(link!=null&&link.length>3)
				list.add(I18n.func_135052_a(Lib.DESC_INFO+"attachedToDim", link[1],link[2],link[3],link[0]));
		}
	}

	@Override
	public EnumActionResult onItemUseFirst(ItemStack stack, EntityPlayer player, World world, BlockPos pos, EnumFacing side, float hitX, float hitY, float hitZ, EnumHand hand)
	{
		if(!world.field_72995_K)
		{
			TileEntity tileEntity = world.func_175625_s(pos);
			if(tileEntity instanceof IImmersiveConnectable && ((IImmersiveConnectable)tileEntity).canConnect())
			{
				TargetingInfo target = new TargetingInfo(side, hitX,hitY,hitZ);
				WireType wire = getWireType(stack);
				BlockPos masterPos = ((IImmersiveConnectable)tileEntity).getConnectionMaster(wire, target);
				tileEntity = world.func_175625_s(masterPos);
				if( !(tileEntity instanceof IImmersiveConnectable) || !((IImmersiveConnectable)tileEntity).canConnect())
					return EnumActionResult.PASS;

				if( !((IImmersiveConnectable)tileEntity).canConnectCable(wire, target))
				{
					player.func_145747_a(new TextComponentTranslation(Lib.CHAT_WARN+"wrongCable"));
					return EnumActionResult.FAIL;
				}

				if(!ItemNBTHelper.hasKey(stack, "linkingPos"))
				{
					ItemNBTHelper.setIntArray(stack, "linkingPos", new int[]{world.field_73011_w.getDimension(),masterPos.func_177958_n(),masterPos.func_177956_o(),masterPos.func_177952_p()});
					target.writeToNBT(stack.func_77978_p());
				}
				else
				{
					WireType type = getWireType(stack);
					int[] array = ItemNBTHelper.getIntArray(stack, "linkingPos");
					BlockPos linkPos = new BlockPos(array[1],array[2],array[3]);
					TileEntity tileEntityLinkingPos = world.func_175625_s(linkPos);
					int distanceSq = (int) Math.ceil( linkPos.func_177951_i(masterPos) );
					if(array[0]!=world.field_73011_w.getDimension())
						player.func_145747_a(new TextComponentTranslation(Lib.CHAT_WARN+"wrongDimension"));
					else if(linkPos.equals(masterPos))
						player.func_145747_a(new TextComponentTranslation(Lib.CHAT_WARN+"sameConnection"));
					else if( distanceSq > (type.getMaxLength()*type.getMaxLength()))
						player.func_145747_a(new TextComponentTranslation(Lib.CHAT_WARN+"tooFar"));
					else if(!(tileEntityLinkingPos instanceof IImmersiveConnectable))
						player.func_145747_a(new TextComponentTranslation(Lib.CHAT_WARN+"invalidPoint"));
					else
					{
						IImmersiveConnectable nodeHere = (IImmersiveConnectable)tileEntity;
						IImmersiveConnectable nodeLink = (IImmersiveConnectable)tileEntityLinkingPos;
						boolean connectionExists = false;
						Set<Connection> outputs = ImmersiveNetHandler.INSTANCE.getConnections(world, Utils.toCC(nodeHere));
						if(outputs!=null)
							for(Connection con : outputs)
							{
								if(con.end.equals(Utils.toCC(nodeLink)))
									connectionExists = true;
							}
						if(connectionExists)
							player.func_145747_a(new TextComponentTranslation(Lib.CHAT_WARN+"connectionExists"));
						else
						{
							Vec3d rtOff0 = nodeHere.getRaytraceOffset(nodeLink).func_72441_c(masterPos.func_177958_n(), masterPos.func_177956_o(), masterPos.func_177952_p());
							Vec3d rtOff1 = nodeLink.getRaytraceOffset(nodeHere).func_72441_c(linkPos.func_177958_n(), linkPos.func_177956_o(), linkPos.func_177952_p());
							Set<BlockPos> ignore = new HashSet<>();
							ignore.addAll(nodeHere.getIgnored(nodeLink));
							ignore.addAll(nodeLink.getIgnored(nodeHere));
							boolean canSee = Utils.rayTraceForFirst(rtOff0, rtOff1, world, ignore)==null;
							if(canSee)
							{
								TargetingInfo targetLink = TargetingInfo.readFromNBT(stack.func_77978_p());
								ImmersiveNetHandler.INSTANCE.addConnection(world, Utils.toCC(nodeHere), Utils.toCC(nodeLink), (int)Math.sqrt(distanceSq), type);

								nodeHere.connectCable(type, target, nodeLink);
								nodeLink.connectCable(type, targetLink, nodeHere);
								IESaveData.setDirty(world.field_73011_w.getDimension());
								player.func_71029_a(IEAchievements.connectWire);

								if(!player.field_71075_bZ.field_75098_d)
									stack.field_77994_a--;
								((TileEntity)nodeHere).func_70296_d();
								world.func_175641_c(masterPos, ((TileEntity) nodeHere).func_145838_q(), -1, 0);
								IBlockState state = world.func_180495_p(masterPos);
								world.func_184138_a(masterPos, state,state, 3);
								((TileEntity)nodeLink).func_70296_d();
								world.func_175641_c(linkPos, ((TileEntity) nodeLink).func_145838_q(), -1, 0);
								state = world.func_180495_p(linkPos);
								world.func_184138_a(linkPos, state,state, 3);
							}
							else
								player.func_145747_a(new TextComponentTranslation(Lib.CHAT_WARN+"cantSee"));
						}
					}
					ItemNBTHelper.remove(stack, "linkingPos");
					ItemNBTHelper.remove(stack, "side");
					ItemNBTHelper.remove(stack, "hitX");
					ItemNBTHelper.remove(stack, "hitY");
					ItemNBTHelper.remove(stack, "hitZ");
				}
				return EnumActionResult.SUCCESS;
			}
		}
		return EnumActionResult.PASS;
	}
}
